/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Arrays;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import com.koutra.dist.proc.faucet.ReaderFaucet;
import com.koutra.dist.proc.faucet.StreamFaucet;
import com.koutra.dist.proc.faucet.XMLFaucet;
import com.koutra.dist.proc.pipeline.IdentityReaderToWriterPipelineItemTemplate;
import com.koutra.dist.proc.pipeline.IdentityStreamToStreamPipelineItemTemplate;
import com.koutra.dist.proc.pipeline.ReaderDemuxPipelineItem;
import com.koutra.dist.proc.pipeline.ReaderTaskBoundaryTemplate;
import com.koutra.dist.proc.pipeline.StreamDemuxPipelineItem;
import com.koutra.dist.proc.pipeline.StreamTaskBoundaryTemplate;
import com.koutra.dist.proc.pipeline.XMLFilterDemuxPipelineItem;
import com.koutra.dist.proc.pipeline.XMLFilterPipelineItemTemplate;
import com.koutra.dist.proc.pipeline.XMLTaskBoundaryTemplate;
import com.koutra.dist.proc.sink.StreamMuxSink;
import com.koutra.dist.proc.sink.WriterMuxSink;
import com.koutra.dist.proc.sink.XMLZipMuxSink;
import com.koutra.dist.proc.util.Util;
import com.koutra.dist.proc.xml.XPathXMLMuxFilter;

import junit.framework.TestCase;

public class ClusterTaskTest extends TestCase {

	public void testStreamTask() throws IOException {
		StreamFaucet faucet = new StreamFaucet("1", "test/input.txt");
		StreamTaskBoundaryTemplate template1 = new StreamTaskBoundaryTemplate("3");
		StreamDemuxPipelineItem demux = new StreamDemuxPipelineItem("2", template1) {
			@Override
			public boolean shouldChangePipeline() {
				return false;
			}
			
			@Override
			public byte[] transformBuffer(byte[] buffer, int off, int len) {
				if (len == -1)
					return new byte[0];
				return Arrays.copyOfRange(buffer, off, len);
			}
		};
		IdentityStreamToStreamPipelineItemTemplate template2 =
			new IdentityStreamToStreamPipelineItemTemplate();
		StreamTaskBoundaryTemplate template3 = new StreamTaskBoundaryTemplate("4");
		StreamMuxSink sink = new StreamMuxSink("5", "test/output.txt");
		
		// Hook everything up
		faucet.hookupSink(demux);
		demux.hookupSink(template1);
		template1.hookupSink(template2);
		template2.hookupSink(template3);
		sink.hookupFaucet(template3);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		InputStream inputFileIS = new FileInputStream("test/input.txt");
		InputStream outputFileIS = new FileInputStream("test/output.txt");
		Util.testStreamEquivalence(inputFileIS, outputFileIS);
		inputFileIS.close();
		outputFileIS.close();
		
		File f = new File("test/output.txt");
		f.delete();
	}
	
	public void testReaderTask() throws IOException {
		ReaderFaucet faucet = new ReaderFaucet("1", "test/greek_input.txt", "UTF-8");
		ReaderTaskBoundaryTemplate template1 = new ReaderTaskBoundaryTemplate("3");
		ReaderDemuxPipelineItem demux = new ReaderDemuxPipelineItem("2", template1) {
			@Override
			public boolean shouldChangePipeline() {
				return false;
			}
			
			@Override
			public char[] transformBuffer(char[] buffer, int off, int len) {
				if (len == -1)
					return new char[0];
				return Arrays.copyOfRange(buffer, off, len);
			}
		};
		IdentityReaderToWriterPipelineItemTemplate template2 =
			new IdentityReaderToWriterPipelineItemTemplate();
		ReaderTaskBoundaryTemplate template3 = new ReaderTaskBoundaryTemplate("4");
		WriterMuxSink sink = new WriterMuxSink("5", "test/greek_output.txt", "Cp1253");
		
		// Hook everything up
		faucet.hookupSink(demux);
		demux.hookupSink(template1);
		template1.hookupSink(template2);
		template2.hookupSink(template3);
		sink.hookupFaucet(template3);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		Reader inputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_input.txt"), "UTF-8");
		Reader outputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_output.txt"), "Cp1253");
		Util.testReaderEquivalence(inputFileReader, outputFileReader);
		inputFileReader.close();
		outputFileReader.close();
		
		File fileToDelete = new File("test/greek_output.txt");
		fileToDelete.delete();
	}

	public void testXMLTask() throws IOException {
		XMLFaucet faucet = new XMLFaucet("1", "test/input.xml");
		XMLTaskBoundaryTemplate template1 = new XMLTaskBoundaryTemplate("3");
		XMLFilterDemuxPipelineItem demux = new XMLFilterDemuxPipelineItem(
				"2", new XPathXMLMuxFilter("/Article/Sect1"), template1);
		XMLFilterPipelineItemTemplate template2 =
			new XMLFilterPipelineItemTemplate("test/no_xform.xsl");
		XMLTaskBoundaryTemplate template3 = new XMLTaskBoundaryTemplate("4");
		XMLZipMuxSink sink = new XMLZipMuxSink("5", "test/output.zip");
		
		// Hook everything up
		faucet.hookupSink(demux);
		demux.hookupSink(template1);
		template1.hookupSink(template2);
		template2.hookupSink(template3);
		sink.hookupFaucet(template3);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Validate the result
		ZipInputStream zis = new ZipInputStream(new FileInputStream("test/output.zip"));
		ZipEntry firstEntry = zis.getNextEntry();
		assertNotNull(firstEntry);
		InputStream expectedIS = new FileInputStream(
				"test/validation/ClusterTaskTest/testXMLTask/firstZipEntry.xml");
		Util.testStreamEquivalence(zis, expectedIS);
		expectedIS.close();
		ZipEntry secondEntry = zis.getNextEntry();
		assertNotNull(secondEntry);
		expectedIS = new FileInputStream(
				"test/validation/ClusterTaskTest/testXMLTask/secondZipEntry.xml");
		Util.testStreamEquivalence(zis, expectedIS);
		expectedIS.close();
		assertNull(zis.getNextEntry());
		zis.close();

		File fileToDelete = new File("test/output.zip");
		fileToDelete.delete();
	}

}
