/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import junit.framework.TestCase;

import org.apache.fop.apps.MimeConstants;

import com.koutra.dist.proc.faucet.ReaderFaucet;
import com.koutra.dist.proc.faucet.StreamFaucet;
import com.koutra.dist.proc.faucet.XMLFaucet;
import com.koutra.dist.proc.pipeline.IdentityReaderToWriterPipelineItem;
import com.koutra.dist.proc.pipeline.IdentityStreamToStreamPipelineItem;
import com.koutra.dist.proc.pipeline.LineBasedReaderToWriterPipelineItem;
import com.koutra.dist.proc.pipeline.XMLFilterPipelineItem;
import com.koutra.dist.proc.sink.FOPSink;
import com.koutra.dist.proc.sink.StreamSink;
import com.koutra.dist.proc.sink.WriterSink;
import com.koutra.dist.proc.sink.XMLSink;
import com.koutra.dist.proc.util.Util;

public class SimpleTest extends TestCase {
	public void testSimpleStream() throws IOException {
		// Set up the stream objects
		StreamFaucet faucet = new StreamFaucet("1", "test/input.txt");
		IdentityStreamToStreamPipelineItem transformer =
			new IdentityStreamToStreamPipelineItem("2");
		StreamSink sink = new StreamSink("2", "test/output.txt");
		
		// Hook everything up
		transformer.hookupFaucet(faucet);
		transformer.hookupSink(sink);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		InputStream inputFileIS = new FileInputStream("test/input.txt");
		InputStream outputFileIS = new FileInputStream("test/output.txt");
		Util.testStreamEquivalence(inputFileIS, outputFileIS);
		inputFileIS.close();
		outputFileIS.close();
		
		File fileToDelete = new File("test/output.txt");
		fileToDelete.delete();
	}
	
	public void testSimpleCharStream() throws IOException {
		// Set up the transformation
		ReaderFaucet faucet = new ReaderFaucet("1", "test/greek_input.txt", "UTF-8");
		IdentityReaderToWriterPipelineItem transformer =
			new IdentityReaderToWriterPipelineItem("2");
		WriterSink sink = new WriterSink("2", "test/greek_output.txt", "Cp1253");
		
		// Hook everything up
		transformer.hookupFaucet(faucet);
		transformer.hookupSink(sink);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		Reader inputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_input.txt"), "UTF-8");
		Reader outputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_output.txt"), "Cp1253");
		Util.testReaderEquivalence(inputFileReader, outputFileReader);
		inputFileReader.close();
		outputFileReader.close();
		
		File fileToDelete = new File("test/greek_output.txt");
		fileToDelete.delete();
	}
	
	public void testSimpleLineTransform() throws IOException {
		// Set up the transformation
		ReaderFaucet faucet = new ReaderFaucet("1", "test/greek_input.txt", "UTF-8");
		LineBasedReaderToWriterPipelineItem transformer =
				new LineBasedReaderToWriterPipelineItem("2") {
			@Override
			public String transformLine(String line) {
				if (line == null)
					return "";
				return line + "\r\n";
			}
		};
		WriterSink sink = new WriterSink("3", "test/greek_output.txt", "Cp1253");
		
		// Hook everything up
		transformer.hookupFaucet(faucet);
		transformer.hookupSink(sink);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		Reader inputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_input.txt"), "UTF-8");
		Reader outputFileReader = new InputStreamReader(
				new FileInputStream("test/greek_output.txt"), "Cp1253");
		Util.testReaderEquivalence(inputFileReader, outputFileReader);
		inputFileReader.close();
		outputFileReader.close();
		
		File fileToDelete = new File("test/greek_output.txt");
		fileToDelete.delete();
	}
	
	public void testSimpleXMLTransform() throws IOException {
		XMLFaucet faucet = new XMLFaucet("1", "test/input.xml");
		XMLFilterPipelineItem transformerA = new XMLFilterPipelineItem(
				"2", "test/docbookToArticle.xml");
		XMLFilterPipelineItem transformerB = new XMLFilterPipelineItem(
				"3", "test/articleToHtml.xml");
		XMLSink sink = new XMLSink("4", "test/xform_output.xml");
		
		// Hook everything up
		transformerA.hookupFaucet(faucet);
		transformerA.hookupSink(transformerB);
		transformerB.hookupSink(sink);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		
		// Verify the result
		InputStream actualResultIS = new FileInputStream("test/xform_output.xml");
		InputStream expectedResultIS = new FileInputStream(
				"test/validation/SimpleTest/testSimpleXMLTransform/expected_xform_output.xml");
		Util.testStreamEquivalence(actualResultIS, expectedResultIS);
		actualResultIS.close();
		expectedResultIS.close();
		
		File fileToDelete = new File("test/xform_output.xml");
		fileToDelete.delete();
	}
	
	public void testFoTransform() throws IOException {
		XMLFaucet faucet = new XMLFaucet("1", "test/projectteam.xml");
		XMLFilterPipelineItem transformerA = new XMLFilterPipelineItem(
				"2", "test/projectteam2fo.xsl");
		FOPSink sink = new FOPSink("3", MimeConstants.MIME_PDF, "test/projectteam.pdf");
		
		// Hook everything up
		transformerA.hookupFaucet(faucet);
		transformerA.hookupSink(sink);
		
		// Start the ball rolling
		sink.consume();
		
		// Shut things down
		sink.dispose();
		transformerA.dispose();
		faucet.dispose();
		
		// Verify the result
		InputStream actualResultIS = new FileInputStream("test/projectteam.pdf");
		InputStream expectedResultIS = new FileInputStream(
				"test/validation/SimpleTest/testFoTransform/projectteam.pdf");
		Util.testPDFEquivalence(actualResultIS, expectedResultIS);
		actualResultIS.close();
		expectedResultIS.close();
		
		File fileToDelete = new File("test/projectteam.pdf");
		fileToDelete.delete();
	}

}
