/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.cluster;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.jgroups.Address;
import org.jgroups.util.Streamable;

/**
 * A task ID uniquely identifies a task in the cluster of cooperating nodes.
 * 
 * @author Pafsanias Ftakas
 */
public class TaskID implements Streamable {
	
	private static int next_id = 0;
	
	private Address address;
	private int id;
	
	/**
	 * @deprecated use one of the static createTaskID() methods
	 */
	public TaskID() {
		this.address = null;
		this.id = 0;
	}
	
	private TaskID(Address address, int id) {
		this.address = address;
		this.id = id;
	}
	
	public static synchronized TaskID createTaskID(int executorRank,
			int clusterSize, Address address) {
		if (next_id % clusterSize == executorRank)
			next_id++;
		return new TaskID(address, next_id++);
	}
	
	public static TaskID createTaskIDFromStream(DataInputStream in)
			throws IOException, IllegalAccessException, InstantiationException {
		TaskID retVal = new TaskID(null, 0);
		retVal.readFrom(in);
		return retVal;
	}
	
	public Address getAddress() {
		return address;
	}

	public int getId() {
		return id;
	}

	@Override
	public String toString() {
		return "TaskID:[" + address + "," + id + "]";
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		
		if (!(obj instanceof TaskID))
			return false;
		
		TaskID other = (TaskID) obj;
		return address.equals(other.address) && id == other.id;
	}
	
	@Override
	public int hashCode() {
		return address.hashCode() + id;
	}

	@Override
	public void readFrom(DataInputStream in) throws IOException,
			IllegalAccessException, InstantiationException {
		String clazz = in.readUTF();
		try {
			this.address = (Address) Class.forName(clazz).newInstance();
		} catch (Exception e) {
			throw new InstantiationException("Unable to create instance of: " +
					clazz + " because of: " + e.getMessage());
		}
		this.address.readFrom(in);
		this.id = in.readInt();
	}

	@Override
	public void writeTo(DataOutputStream out) throws IOException {
		out.writeUTF(address.getClass().getCanonicalName());
		address.writeTo(out);
		out.writeInt(id);
	}

}
