/*******************************************************************************
 * Copyright (c) 2009 Pafsanias Ftakas.
 *
 * This file is part of DistProc.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * DistProc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Eclipse Public License v1.0 for more details.
 *
 * Contributors:
 *     Pafsanias Ftakas - initial API and implementation
 *******************************************************************************/
package com.koutra.dist.proc.sink;

import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;

import com.koutra.dist.proc.model.ContentType;
import com.koutra.dist.proc.model.IDemuxFaucet;
import com.koutra.dist.proc.model.IFaucet;
import com.koutra.dist.proc.model.IFaucetTemplate;
import com.koutra.dist.proc.model.IPipelineItem;
import com.koutra.dist.proc.model.ISink;
import com.koutra.dist.proc.model.XformationException;
import com.koutra.dist.proc.util.ExecutorsHelper;

/**
 * A sink implementation that writes out to a writer. When using the
 * constructor that supplies the writer, the caller has the responsibility
 * of closing the writer when the transformation is completed.
 * 
 * @author Pafsanias Ftakas
 */
public class WriterSink implements ISink {
	private static final Logger logger = Logger.getLogger(WriterSink.class);
	
	protected enum Type {
		File,
		Writer;
	};
	
	protected String id;
	protected IFaucet faucet;
	protected boolean hookedUp;
	protected Type type;
	protected Writer writer;
	protected Reader reader;
	protected String path;
	protected String outputCharset;
	
	/**
	 * @deprecated Use any of the initializing constructors instead.
	 */
	public WriterSink() { }
	
	public WriterSink(String id, Writer writer) {
		this.id = id;
		this.faucet = null;
		this.hookedUp = false;
		this.type = Type.Writer;
		this.writer = writer;
		this.reader = null;
		this.path = null;
		this.outputCharset = null;
	}

	public WriterSink(String id, String path, String outputCharset) {
		this.id = id;
		this.faucet = null;
		this.hookedUp = false;
		this.type = Type.File;
		this.writer = null;
		this.reader = null;
		this.path = path;
		this.outputCharset = outputCharset;
	}

	@Override
	public String getId() {
		return id;
	}

	@Override
	public IFaucet getFaucet() {
		return faucet;
	}

	@Override
	public boolean supportsInput(ContentType contentType) {
		switch (contentType) {
		case CharStream:
			return true;
		case ByteStream:
		case XML:
		case ResultSet:
		default:
			return false;
		}
	}

	@Override
	public void hookupFaucet(IFaucet faucet) {
		if (!faucet.supportsOutput(ContentType.CharStream))
			throw new IllegalArgumentException("Faucet '" + faucet.getId() +
					"' must support the CharStream content type.");
		if (faucet instanceof IDemuxFaucet)
			throw new IllegalArgumentException("Faucet '" + faucet.getId() +
					"' is an IDemuxFaucet instance. This is not allowed.");
		if (faucet instanceof IFaucetTemplate)
			throw new IllegalArgumentException("Faucet '" + faucet.getId() +
					"' is an IFaucetTemplate instance. This is not allowed.");
		if (hookedUp && this.faucet == faucet) {
			// Effectively a NO-OP.
			return;
		}
		if (hookedUp)
			throw new XformationException("Trying to hook up an already hooked up sink.");
		
		this.faucet = faucet;
		this.hookedUp = true;
		
		this.faucet.hookupSink(this);
	}
	
	@Override
	public void registerSource(Object source) {
		// Only used in mux/demux situations.
	}

	@Override
	public String dumpPipeline() {
		return getClass().getName() + ": " + reader + "->" + writer;
	}

	@Override
	public void dispose() {
		switch (type) {
		case File:
			try {
				writer.close();
			} catch (IOException e) {
				throw new XformationException("Unable to close writer", e);
			}
			break;
		case Writer:
			// Caller owns the writer. It is their responsibility to close it.
			break;
		}
		
		faucet.dispose();
	}

	@Override
	public void consume() {
		if (!hookedUp)
			throw new XformationException("Sink has not been set up correctly: " +
					"faucet has not been set");
		
		switch (type) {
		case File:
			try {
				if (outputCharset == null)
					writer = new BufferedWriter(new FileWriter(path));
				else
					writer = new BufferedWriter(new OutputStreamWriter(
							new FileOutputStream(path), outputCharset));
			} catch (IOException e) {
				throw new XformationException("Unable to create output writer", e);
			}
			break;
		case Writer:
			break;
		}
		
		if (faucet instanceof IPipelineItem) {
			((IPipelineItem) faucet).consume(this);
		}
		
		try {
			reader = (Reader) faucet.getSource(ContentType.CharStream);
			int count;
			char[] buffer = new char[8 * 1024];
			if (logger.isTraceEnabled())
				logger.trace("Using the reader " + reader + " in the sink");
			while ((count = reader.read(buffer)) != -1) {
				if (logger.isTraceEnabled())
					logger.trace("Read '" + new String(buffer, 0, count) +
							"' from reader " + reader + " in the sink");
				writer.write(buffer, 0, count);
			}
		} catch (IOException ioe) {
			throw new XformationException("Unable to transform char stream", ioe);
		}
	}

	@Override
	public Future<?> consumeAsynchronously() {
		return ExecutorsHelper.getInstance().submitInProc(new Runnable() {
			@Override
			public void run() {
				try {
					consume();
				} catch (Throwable t) {
					logger.error("Unable to consume faucet", t);
				}
			}
		});
	}

	@Override
	public void readFrom(DataInputStream in) throws IOException,
			IllegalAccessException, InstantiationException {
		this.id = in.readUTF();
		this.faucet = null;
		this.hookedUp = false;
		this.type = Type.File;
		this.writer = null;
		this.reader = null;
		this.path = in.readUTF();
		this.outputCharset = in.readUTF();
	}

	@Override
	public void writeTo(DataOutputStream out) throws IOException {
		if (type != Type.File)
			throw new XformationException(
					"Only WriterSinks of type Type.File can be streamed.");
		
		out.writeUTF(id);
		out.writeUTF(path);
		out.writeUTF(outputCharset);
	}

}
